package com.bosch.tt.rnmedusa.ui.components;

import android.content.Context;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.view.animation.Animation;
import android.view.animation.AnimationUtils;
import android.widget.FrameLayout;

import com.bosch.tt.rnmedusa.ui.R;
import com.facebook.react.bridge.ReadableArray;

public class ActivityIndicator extends FrameLayout implements Animation.AnimationListener {
	private int boxSize;

	private int[] mTopColors;
	private int[] mBottomColors;
	private int[] mMiddleColors;
	private int mTopColorIndex;
	private int mBottomColorIndex;
	private int mMiddleColorIndex;

	private View mTopLeftRect;
	private View mBottomRightRect;
	private View mMiddleRect;

	private boolean mAnimating;
	private Animation mFadeIn;
	private Animation mFadeOut;
	private Animation mAnimLeftOut;
	private Animation mAnimLeftIn;
	private Animation mAnimRightOut;
	private Animation mAnimTopOut;
	private Animation mScaleVerticalIn;
	private Animation mAnimBottomOut;
	private Animation mScaleHorizontalOut;
	private Animation mAnimRightIn;
	private Animation mAnimTopIn;
	private Animation mAnimBottomIn;
	private Animation mScaleVerticalOut;
	private Animation mScaleHorizontalIn;

	public ActivityIndicator(Context context) {
		super(context);

		LayoutInflater inflater = LayoutInflater.from(context);
		inflater.inflate(R.layout.activity_indicator, this);

		mTopLeftRect = findViewById(R.id.top_left_rect);
		mBottomRightRect = findViewById(R.id.bottom_right_rect);
		mMiddleRect = findViewById(R.id.middle_rect);
	}

	public void setBoxSize(int boxSize) {
		this.boxSize = boxSize;
		initialize();
	}

	public void setTopColors(ReadableArray topColors) {
		this.mTopColors = convertColors(topColors);
	}

	public void setBottomColors(ReadableArray bottomColors) {
		this.mBottomColors = convertColors(bottomColors);
	}

	public void setMiddleColors(ReadableArray middleColors) {
		this.mMiddleColors = convertColors(middleColors);
	}

	public void startAnimation() {
		if (this.mAnimating) {
			return;
		}

		this.mAnimating = true;

		this.mMiddleRect.setVisibility(View.VISIBLE);
		this.mTopLeftRect.setVisibility(View.VISIBLE);
		this.mBottomRightRect.setVisibility(View.VISIBLE);

		this.mTopColorIndex = 0;
		this.mBottomColorIndex = 0;
		this.mMiddleColorIndex = 0;
		updateColors();

		// NOTE: bottom color should not change after animation start, that's why -1 is needed here
		this.mBottomColorIndex = -1;

		startHorizontalOut();
	}

	public void endAnimation() {
		this.mAnimating = false;

		this.mTopLeftRect.clearAnimation();
		this.mBottomRightRect.clearAnimation();
		this.mMiddleRect.clearAnimation();
	}

	private void initialize() {
		initRect(mTopLeftRect);
		initRect(mBottomRightRect);
		initRect(mMiddleRect);
	}

	private void initRect(View rect) {
		float density = getContext().getResources().getDisplayMetrics().density;
		int boxSizePx = Math.round((float) boxSize * density);

		ViewGroup.LayoutParams topLayoutParams = rect.getLayoutParams();
		topLayoutParams.width = boxSizePx;
		topLayoutParams.height = boxSizePx;
		rect.setLayoutParams(topLayoutParams);
	}


	@Override
	public void onAnimationStart(Animation animation) {

	}

	@Override
	public void onAnimationEnd(Animation animation) {
		if (!this.mAnimating) {
			return;
		}

		if (animation == this.mAnimTopOut) {
			startVerticalIn();
		} else if (animation == this.mAnimTopIn) {
			startHorizontalOut();
		} else if (animation == this.mAnimLeftOut) {
			startHorizontalIn();
		} else if (animation == this.mAnimLeftIn) {
			startVerticalOut();
		}

	}

	@Override
	public void onAnimationRepeat(Animation animation) {

	}

	private void startHorizontalOut() {
		animateHorizontalOut();
		animateMiddleHorizontalOut();
	}

	private void startHorizontalIn() {
		animateHorizontalIn();
		animateMiddleHorizontalIn();
	}

	private void startVerticalOut() {
		animateVerticalOut();
		animateMiddleVerticalOut();
	}

	private void startVerticalIn() {
		animateVerticalIn();
		animateMiddleVerticalIn();
	}

	private void animateHorizontalOut() {
		this.mBottomColorIndex = (this.mBottomColorIndex + 1) % this.mBottomColors.length;
		updateColors();

		this.mTopLeftRect.startAnimation(getAnimLeftOut());
		this.mBottomRightRect.startAnimation(getAnimRightOut());
	}

	private void animateHorizontalIn() {
		this.mMiddleColorIndex = (this.mMiddleColorIndex + 1) % this.mMiddleColors.length;
		updateColors();

		this.mTopLeftRect.startAnimation(getAnimLeftIn());
		this.mBottomRightRect.startAnimation(getAnimRightIn());
	}

	private void animateVerticalOut() {
		this.mTopColorIndex = (this.mTopColorIndex + 1) % this.mTopColors.length;
		updateColors();

		this.mBottomRightRect.startAnimation(getAnimBottomOut());
		this.mTopLeftRect.startAnimation(getAnimTopOut());
	}

	private void animateVerticalIn() {
		this.mMiddleColorIndex = (this.mMiddleColorIndex + 1) % this.mMiddleColors.length;
		updateColors();

		this.mBottomRightRect.startAnimation(getAnimBottomIn());
		this.mTopLeftRect.startAnimation(getAnimTopIn());
	}

	private void animateMiddleHorizontalOut() {
		this.mMiddleRect.startAnimation(getScaleHorizontalOut());
	}
	private void animateMiddleHorizontalIn() {
		this.mMiddleRect.startAnimation(getScaleHorizontalIn());
	}

	private void animateMiddleVerticalOut() {
		this.mMiddleRect.startAnimation(getScaleVerticalOut());
	}
	private void animateMiddleVerticalIn() {
		this.mMiddleRect.startAnimation(getScaleVerticalIn());
	}

	private Animation getAnimLeftOut() {
		if (this.mAnimLeftOut == null) {
			this.mAnimLeftOut = AnimationUtils.loadAnimation(getContext(), R.anim.loader_translate_left_out);
			this.mAnimLeftOut.setAnimationListener(this);
		}
		return this.mAnimLeftOut;
	}

	private Animation getAnimLeftIn() {
		if (this.mAnimLeftIn == null) {
			this.mAnimLeftIn = AnimationUtils.loadAnimation(getContext(), R.anim.loader_translate_left_in);
			this.mAnimLeftIn.setAnimationListener(this);
		}
		return this.mAnimLeftIn;
	}

	private Animation getAnimRightOut() {
		if (this.mAnimRightOut == null) {
			this.mAnimRightOut = AnimationUtils.loadAnimation(getContext(), R.anim.loader_translate_right_out);
		}
		return this.mAnimRightOut;
	}

	private Animation getAnimRightIn() {
		if (this.mAnimRightIn == null) {
			this.mAnimRightIn = AnimationUtils.loadAnimation(getContext(), R.anim.loader_translate_right_in);
		}
		return this.mAnimRightIn;
	}

	private Animation getAnimTopOut() {
		if (this.mAnimTopOut == null) {
			this.mAnimTopOut = AnimationUtils.loadAnimation(getContext(), R.anim.loader_translate_top_out);
			this.mAnimTopOut.setAnimationListener(this);
		}
		return this.mAnimTopOut;
	}

	private Animation getAnimTopIn() {
		if (this.mAnimTopIn == null) {
			this.mAnimTopIn = AnimationUtils.loadAnimation(getContext(), R.anim.loader_translate_top_in);
			this.mAnimTopIn.setAnimationListener(this);
		}
		return this.mAnimTopIn;
	}

	private Animation getAnimBottomOut() {
		if (this.mAnimBottomOut == null) {
			this.mAnimBottomOut = AnimationUtils.loadAnimation(getContext(), R.anim.loader_translate_bottom_out);
		}
		return this.mAnimBottomOut;
	}

	private Animation getAnimBottomIn() {
		if (this.mAnimBottomIn == null) {
			this.mAnimBottomIn = AnimationUtils.loadAnimation(getContext(), R.anim.loader_translate_bottom_in);
		}
		return this.mAnimBottomIn;
	}

	private Animation getScaleVerticalIn() {
		if (this.mScaleVerticalIn == null) {
			this.mScaleVerticalIn = AnimationUtils.loadAnimation(getContext(), R.anim.loader_scale_vertical_in);
		}
		return this.mScaleVerticalIn;
	}
	private Animation getScaleVerticalOut() {
		if (this.mScaleVerticalOut == null) {
			this.mScaleVerticalOut = AnimationUtils.loadAnimation(getContext(), R.anim.loader_scale_vertical_out);
		}
		return this.mScaleVerticalOut;
	}

	private Animation getScaleHorizontalOut() {
		if (this.mScaleHorizontalOut == null) {
			this.mScaleHorizontalOut = AnimationUtils.loadAnimation(getContext(), R.anim.loader_scale_horizontal_out);
		}
		return this.mScaleHorizontalOut;
	}

	private Animation getScaleHorizontalIn() {
		if (this.mScaleHorizontalIn == null) {
			this.mScaleHorizontalIn = AnimationUtils.loadAnimation(getContext(), R.anim.loader_scale_horizontal_in);
		}
		return this.mScaleHorizontalIn;
	}

	private void updateColors() {
		this.mTopLeftRect.setBackgroundColor(this.mTopColors[this.mTopColorIndex]);
		this.mBottomRightRect.setBackgroundColor(this.mBottomColors[this.mBottomColorIndex]);
		this.mMiddleRect.setBackgroundColor(this.mMiddleColors[this.mMiddleColorIndex]);
	}

	private int[] convertColors(ReadableArray colors) {
		int[] convertedColors = new int[colors.size()];

		for (int i = 0; i < convertedColors.length; i++) {
			convertedColors[i] = colors.getInt(i);
		}

		return convertedColors;
	}
}
