//
//  OfflineValueProvider.m
//  RNPrivacyComponent
//
//  Created by István Bege on 29/10/2018.
//  Copyright (C) 2018, Bosch Thermotechnik GmbH. All rights reserved.
//

#import "OfflineValueProvider.h"
#import "Constants.h"

@interface OfflineValueProvider ()

@property (readonly, nonatomic) NSString* folderName;
@property (readonly, nonatomic) NSString* filePrefix;

@end


@implementation OfflineValueProvider

- (instancetype)initWithFolderName:(NSString *)folderName filePrefix:(NSString *)filePrefix
{
    self = [super init];
    if (self) {
        _folderName = folderName;
        _filePrefix = filePrefix;
    }
    return self;
}

- (TTPResponse *)getDocumentWithCountry:(NSString *)country language:(NSString *)language
{
    NSString *fullPath = [NSString stringWithFormat:@"%@/%@", _folderName, [self getDocumentNameForCode:country andLanguage:language]];
    NSError *error;
    TTPResponse *response;
    NSDictionary *content = [self JSONFromFile:fullPath error:&error];
    if (error != nil) {
        response = [self createErrorResponseWithMessage:error.description];
    } else {
        NSData *jsonData = [NSJSONSerialization dataWithJSONObject:content options:NSJSONWritingPrettyPrinted error:&error];
        if (error != nil) {
            response = [self createErrorResponseWithMessage:error.description];
        } else {
            response = [self createSuccessResponseWithJSON:[[NSString alloc] initWithData:jsonData encoding:NSUTF8StringEncoding]];
        }
    }
    return response;
}

- (TTPResponse *)getAvailableDocuments
{
    TTPResponse *response;
    NSString *fullPath = [[[NSBundle mainBundle] resourcePath] stringByAppendingPathComponent:_folderName];
    BOOL isDirectory;
    // Check if "privacy-policy directory exists"
    if ([[NSFileManager defaultManager] fileExistsAtPath:fullPath isDirectory:&isDirectory] && isDirectory) {
        NSError *error;
        // Retrieving all files from privacy-policy directory
        NSArray *files = [[NSFileManager defaultManager] contentsOfDirectoryAtPath:fullPath error:&error];
        if (error != nil) {
            response = [self createErrorResponseWithMessage:error.description];
        } else {
            NSMutableArray *jsonArray = [[NSMutableArray alloc] init];
            // Read all the files and store their content
            for (id file in files) {
                NSString *fullPath = [NSString stringWithFormat:@"%@/%@", _folderName, file];
                [jsonArray addObject:[self JSONFromFile:fullPath error:&error]];
            }
            // Convert JSON array to NSData
            NSData *jsonData = [NSJSONSerialization dataWithJSONObject:jsonArray options:NSJSONWritingPrettyPrinted error:&error];
            if (error != nil) {
                response = [self createErrorResponseWithMessage:error.description];
            } else {
                // Convert NSData to NSString
                NSString *jsonString = [[NSString alloc] initWithData:jsonData encoding:NSUTF8StringEncoding];
                response = [self createSuccessResponseWithJSON:jsonString];
            }
        }
    } else {
        response = [self createErrorResponseWithMessage:[NSString stringWithFormat:@"%@ directory does not exist.", _folderName]];
    }
    return response;
}

#pragma mark - Private methods

// e.g. privacy-policy_de_de-de.json
- (NSString *)getDocumentNameForCode:(NSString*)code andLanguage:(NSString*)language {
    return [NSString stringWithFormat:@"%@_%@_%@.json", _filePrefix, [code lowercaseString], [language lowercaseString]];
}

- (NSDictionary *)JSONFromFile:(NSString*)file error:(NSError **)error {
    NSString *fileName = [file stringByDeletingPathExtension];
    NSString *extension = [file pathExtension];
    NSString *path = [[NSBundle mainBundle] pathForResource:fileName ofType:extension];
    
    // check if the file exists
    if ([[NSFileManager defaultManager] fileExistsAtPath:path]) {
        NSData *data = [NSData dataWithContentsOfFile:path options:kNilOptions error:error];
        return [NSJSONSerialization JSONObjectWithData:data options:kNilOptions error:error];
    }

    // return error if file does not exist
    NSDictionary *userInfo = @{ ERROR_MSG_KEY: ERROR_FILE_NOT_FOUND_MSG};
    *error = [NSError errorWithDomain:ERROR_DOMAIN code:ERROR_CODE userInfo:userInfo];
    return userInfo;
}

- (TTPResponse *)createErrorResponseWithMessage:(NSString *)errorMessage {
    TTPResponse *response = [TTPResponse new];
    [response setSuccess:NO];
    [response setError:errorMessage];
    [response setJson:nil];
    return response;
}

- (TTPResponse *)createSuccessResponseWithJSON:(NSString *)json {
    TTPResponse *response = [TTPResponse new];
    [response setSuccess:YES];
    [response setError:nil];
    [response setJson:json];
    return response;
}

@end
